@@ -1,5 +1,6 @@
 use crate::*;
 use page::PAGE_SIZE;
+use std::cmp::min;
 use std::collections::hash_map::Entry;
 
 /// Sorted Record storage.
@@ -76,6 +77,31 @@ impl SortedFile {
         self.ok.set(false);
     }
 
+    /// Get the set of used logical pages.
+    pub fn get_used(&self, db: &DB, to: &mut HashSet<u64>) {
+        self.get_used_page(db, to, self.root_page);
+    }
+
+    fn get_used_page(&self, db: &DB, to: &mut HashSet<u64>, pnum: u64) {
+        assert!(to.insert(pnum));
+        let pp = self.load_page(db, pnum);
+        let p = &pp.borrow();
+        if p.level != 0 {
+            self.get_used_page(db, to, p.first_page);
+            self.get_used_pages(db, to, p, p.root);
+        }
+    }
+
+    fn get_used_pages(&self, db: &DB, to: &mut HashSet<u64>, p: &Page, x: usize) {
+        if x != 0 {
+            self.get_used_pages(db, to, p, p.left(x));
+            self.get_used_pages(db, to, p, p.right(x));
+            if p.level > 0 {
+                self.get_used_page(db, to, p.child_page(x));
+            }
+        }
+    }
+
     /// Insert a Record. If the key is a duplicate, the record is not saved.
     pub fn insert(&self, db: &DB, r: &dyn Record) {
         while !self.insert_leaf(db, self.root_page, r, None) {
@@ -331,50 +357,56 @@ impl SortedFile {
 
     /// Attempt to free up logical pages by re-packing child pages.
     pub fn repack(&self, db: &DB, r: &dyn Record) -> i64 {
-        self.repack_page(db, self.root_page, r)
+        let mut freed = 0;
+        self.repack_page(db, self.root_page, r, &mut freed);
+        freed
     }
 
-    /// Repack a page. Result is number of pages freed.
-    fn repack_page(&self, db: &DB, pnum: u64, r: &dyn Record) -> i64 {
-        let mut result = 0;
+    /* Notes on repacking.
+       When repacking a page of level 2 or more, no keys are dropped or created.
+       Instead keys move from the level 1 pages to the level 2 page or vice versa.
+       When repacking a level 1 page, parent keys are dropped, and new keys may be created.
+    */
 
+    /// Repack a page. Result is number of pages freed.
+    fn repack_page(&self, db: &DB, pnum: u64, r: &dyn Record, freed: &mut i64) {
         let pp = self.load_page(db, pnum);
         let p = &mut pp.borrow_mut();
 
         if p.level == 0 {
-            return 0;
+            return;
         }
 
         if p.level > 1 {
-            result += self.repack_page(db, p.first_page, r);
-            if result >= REPACK_LIMIT {
-                return result;
+            self.repack_page(db, p.first_page, r, freed);
+            if *freed >= REPACK_LIMIT {
+                return;
             }
-            result += self.repack_children(db, p, p.root, r);
-            if result >= REPACK_LIMIT {
-                return result;
+            self.repack_children(db, p, p.root, r, freed);
+            if *freed >= REPACK_LIMIT {
+                return;
             }
-            // return result; // For now, don't attempt to pack higher level pages.
         }
 
         let (x, y) = self.page_total(db, p, p.root, r);
         let n = 1 + x;
         if n < 2 {
-            return result;
+            return;
         }
         let total = y + db.page_size(p.first_page);
         let full = (n * PAGE_SIZE) as u64;
         let space = full - total;
-        if space < full / 10 {
-            return result;
+
+        let div = min(10, n as u64);
+        if space < full / div {
+            return;
         }
 
-        self.set_dirty(p, &pp);
-
-        // Iterate over the page child records. Append them into a list of new pages.
+        // Iterate over the page child records, appnding them into a PageList of new pages.
         let mut plist = PageList::default();
         plist.add(db, p.first_page, r, self, None, 0);
         self.move_children(db, p, p.root, r, &mut plist);
+        let n1 = plist.list.len();
 
         if TRACE_PACK {
             println!(
@@ -387,13 +419,13 @@ impl SortedFile {
                 full - total
             );
             println!(
-                "new #children={} record count={}",
-                plist.list.len(),
+                "new #children={} diff={} record count={}",
+                n1,
+                n - n1,
                 plist.count
             );
         }
-        result += plist.store_to(db, p, self);
-        result
+        *freed += plist.store_to(db, p, self);
     }
 
     /// Count number child pages and their total size, to decide whether to repack a parent page.
@@ -414,29 +446,25 @@ impl SortedFile {
             self.move_children(db, p, p.left(x), r, plist);
             let cp = p.child_page(x);
             plist.add(db, cp, r, self, Some(p), x);
-            if p.level == 1
-            // Not sure if this is the right condition.
-            {
+            if p.level == 1 {
                 p.drop_key(db, x, r);
             }
             self.move_children(db, p, p.right(x), r, plist);
         }
     }
 
-    fn repack_children(&self, db: &DB, p: &Page, x: usize, r: &dyn Record) -> i64 {
-        let mut result = 0;
+    fn repack_children(&self, db: &DB, p: &Page, x: usize, r: &dyn Record, freed: &mut i64) {
         if x != 0 {
-            result += self.repack_page(db, p.child_page(x), r);
-            if result >= REPACK_LIMIT {
-                return result;
+            self.repack_page(db, p.child_page(x), r, freed);
+            if *freed >= REPACK_LIMIT {
+                return;
             }
-            result += self.repack_children(db, p, p.left(x), r);
-            if result >= REPACK_LIMIT {
-                return result;
+            self.repack_children(db, p, p.left(x), r, freed);
+            if *freed >= REPACK_LIMIT {
+                return;
             }
-            result += self.repack_children(db, p, p.right(x), r);
+            self.repack_children(db, p, p.right(x), r, freed);
         }
-        result
     }
 } // end impl SortedFile
 
@@ -738,10 +766,10 @@ struct PageList {
     count: usize,
 }
 
-const TRACE_PACK: bool = false;
+const TRACE_PACK: bool = true;
 
 /// Limit on how many pages to free in one transaction.
-const REPACK_LIMIT: i64 = 10;
+const REPACK_LIMIT: i64 = 100;
 
 impl PageList {
     /// Build new parent page.
@@ -776,6 +804,7 @@ impl PageList {
         result as i64
     }
 
+    /// Add a page to the PageList.
     fn add(
         &mut self,
         db: &DB,
@@ -796,7 +825,7 @@ impl PageList {
         p.pnum = u64::MAX;
 
         if p.level > 0
-        // Need to save first_page. Append the parent key, then fixup the page number.
+        // Need to save p.first_page. Append the parent key, then fixup the page number.
         {
             if let Some(pp) = par {
                 self.append_one(db, pp, px, r);
@@ -809,9 +838,11 @@ impl PageList {
                 ap.set_child_page(x, p.first_page);
             }
         }
+        // Append the page records by recursing from p.root.
         self.append(db, p, p.root, r);
     }
 
+    /// Append page records.
     fn append(&mut self, db: &DB, p: &Page, x: usize, r: &dyn Record) {
         if x != 0 {
             self.append(db, p, p.left(x), r);
@@ -820,16 +851,16 @@ impl PageList {
         }
     }
 
+    /// Append a single page record.
     fn append_one(&mut self, db: &DB, p: &Page, x: usize, r: &dyn Record) {
         self.count += 1;
         let mut ap = &mut self.list[self.cur].0;
         if ap.full() {
-            // start a new page
+            // Start a new page.
             let key = if ap.level == 0 {
-                let key = p.get_key(db, x, r);
-                PKey::Dyn(key)
+                PKey::Dyn(p.get_key(db, x, r))
             } else {
-                PKey::Copy( p.copy(x) )
+                PKey::Copy(p.copy(x))
             };
             self.list.push((p.new_page(), key));
             self.cur += 1;
